import { Authorized, Body, Delete, Get, JsonController, OnUndefined, Param, Post, QueryParam } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { GroupContactNotFoundError } from '../errors/GroupContactErrors';
import { RunnerGroupNotFoundError } from '../errors/RunnerGroupErrors';
import { CreateGroupContact } from '../models/actions/create/CreateGroupContact';
import { GroupContact } from '../models/entities/GroupContact';
import { ResponseEmpty } from '../models/responses/ResponseEmpty';
import { ResponseGroupContact } from '../models/responses/ResponseGroupContact';

@JsonController('/contacts')
@OpenAPI({ security: [{ "AuthToken": [] }, { "RefreshTokenCookie": [] }] })
export class ContactController {
	private contactRepository: Repository<GroupContact>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.contactRepository = getConnectionManager().get().getRepository(GroupContact);
	}

	@Get()
	@Authorized("CONTACT:GET")
	@ResponseSchema(ResponseGroupContact, { isArray: true })
	@OpenAPI({ description: 'Lists all contacts. <br> This includes the contact\'s associated groups.' })
	async getAll() {
		let responseContacts: ResponseGroupContact[] = new Array<ResponseGroupContact>();
		const contacts = await this.contactRepository.find({ relations: ['groups'] });
		contacts.forEach(contact => {
			responseContacts.push(contact.toResponse());
		});
		return responseContacts;
	}

	@Get('/:id')
	@Authorized("CONTACT:GET")
	@ResponseSchema(ResponseGroupContact)
	@ResponseSchema(GroupContactNotFoundError, { statusCode: 404 })
	@OnUndefined(GroupContactNotFoundError)
	@OpenAPI({ description: 'Lists all information about the contact whose id got provided. <br> This includes the contact\'s associated groups.' })
	async getOne(@Param('id') id: number) {
		let contact = await this.contactRepository.findOne({ id: id }, { relations: ['groups'] })
		if (!contact) { throw new GroupContactNotFoundError(); }
		return contact.toResponse();
	}

	@Post()
	@Authorized("CONTACT:CREATE")
	@ResponseSchema(ResponseGroupContact)
	@ResponseSchema(RunnerGroupNotFoundError, { statusCode: 404 })
	@OpenAPI({ description: 'Create a new contact.' })
	async post(@Body({ validate: true }) createContact: CreateGroupContact) {
		let contact;
		try {
			contact = await createContact.toEntity();
		} catch (error) {
			throw error;
		}

		contact = await this.contactRepository.save(contact)
		return (await this.contactRepository.findOne(contact, { relations: ['groups'] })).toResponse();
	}

	// @Put('/:id')
	// @Authorized("DONOR:UPDATE")
	// @ResponseSchema(ResponseDonor)
	// @ResponseSchema(DonorNotFoundError, { statusCode: 404 })
	// @ResponseSchema(DonorIdsNotMatchingError, { statusCode: 406 })
	// @OpenAPI({ description: "Update the contact whose id you provided. <br> Please remember that ids can't be changed." })
	// async put(@Param('id') id: number, @Body({ validate: true }) contact: UpdateDonor) {
	// 	let oldDonor = await this.contactRepository.findOne({ id: id });

	// 	if (!oldDonor) {
	// 		throw new DonorNotFoundError();
	// 	}

	// 	if (oldDonor.id != contact.id) {
	// 		throw new DonorIdsNotMatchingError();
	// 	}

	// 	await this.contactRepository.save(await contact.update(oldDonor));
	// 	return new ResponseDonor(await this.contactRepository.findOne({ id: id }, { relations: ['donations', 'donations.runner', 'donations.runner.scans', 'donations.runner.scans.track'] }));
	// }

	@Delete('/:id')
	@Authorized("CONTACT:DELETE")
	@ResponseSchema(ResponseGroupContact)
	@ResponseSchema(ResponseEmpty, { statusCode: 204 })
	@OnUndefined(204)
	@OpenAPI({ description: 'Delete the contact whose id you provided. <br> If no contact with this id exists it will just return 204(no content). <br> This won\'t delete any groups associated with the contact.' })
	async remove(@Param("id") id: number, @QueryParam("force") force: boolean) {
		let contact = await this.contactRepository.findOne({ id: id });
		if (!contact) { return null; }
		const responseContact = await this.contactRepository.findOne(contact, { relations: ['groups'] });

		await this.contactRepository.delete(contact);
		return responseContact.toResponse();
	}
}
